<?php

namespace TDevAgency\FilesManager\Http\Controllers;

use claviska\SimpleImage;
use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\UploadedFile;
use Illuminate\Routing\Controller as BaseController;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Psr\SimpleCache\InvalidArgumentException;
use TDevAgency\FilesManager\Facades\PictureTag;
use TDevAgency\FilesManager\Helpers\CacheHelper;
use TDevAgency\FilesManager\Helpers\GenerateSizesHelper;
use TDevAgency\FilesManager\Helpers\SymlinkHelper;
use TDevAgency\FilesManager\ResourceModels\TdfmFile;
use Throwable;

class FilesManagerControllerApi extends BaseController
{
    use AuthorizesRequests;
    use DispatchesJobs;
    use ValidatesRequests;

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function store(Request $request): JsonResponse
    {
        $savedFiles = [];
        try {
            DB::transaction(function () use ($request, &$savedFiles) {
                if ($request->hasFile('files')) {
                    $files = $request->file('files');
                    /**
                     * @var UploadedFile $file
                     */
                    foreach ($files as $file) {
                        $filesManager = new TdfmFile();
                        $filesManager->name = Storage::disk('tdfm_original')->put('/', $file);
                        if (
                            pathinfo(
                                $filesManager->name,
                                PATHINFO_EXTENSION
                            ) === '' && $file->getClientOriginalExtension() === 'svg'
                        ) {
                            $newName = sprintf('%s.%s', $filesManager->name, $file->getClientOriginalExtension());
                            Storage::disk('tdfm_original')->move($filesManager->name, $newName);
                            $filesManager->name = $newName;
                        }
                        $savedFiles[] = $filesManager->name;
                        $filesManager->original_name = Str::slug(
                            pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME)
                        ) . '.' . pathinfo($file->getClientOriginalName(), PATHINFO_EXTENSION);
                        $filesManager->file_mime = $file->getClientMimeType();
                        $filesManager->file_size = $file->getSize();
                        $filesManager->save();

                        if (preg_match('/^image\/(jpe?g|gif|png|webp)+$/', $file->getClientMimeType())) {
                            /** @var SimpleImage $imgage */
                            app(GenerateSizesHelper::class)->handle($filesManager);
                        }
                        Artisan::call('fm:symlinks', ['--id' => $filesManager->id]);
                    }
                }
            });

            return $this->index($request);
        } catch (Throwable $e) {
            if (!empty($savedFiles)) {
                foreach ($savedFiles as $file) {
                    if (Storage::disk('tdfm_original')->exists($file)) {
                        Storage::disk('tdfm_original')->delete($file);
                    }
                }
            }
            Log::error($e);
            return JsonResponse::create(App::environment() === 'local' ? [$e->getMessage()] : [], 500);
        }
    }

    /**
     * Display a listing of the resource.
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function index(Request $request): JsonResponse
    {
        $attachments = TdfmFile::sortable()->orderByDesc('created_at');
        if ($request->query->has('q')) {
            $attachments = $attachments->where('original_name', 'like', '%' . request()->query('q') . "%");
        }

        $attachments = $attachments->paginate($request->get('limit') ?? 24);

        return JsonResponse::create($attachments);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param $id
     * @param Request $request
     * @return JsonResponse
     * @throws InvalidArgumentException
     */
    public function destroy($id, Request $request)
    {
        $tdfm = Config::get('tdfm');
        $count = 0;
        foreach ($tdfm['models_with_images'] as $modelName => $row) {
            foreach ($row as $rowModel => $field) {
                $query = $modelName::where($field, '=', $id)->get();
                $count += count($query);
            }
        }
        if ($count !== 0) {
            return JsonResponse::create(['count' => $count]);
        }
        $tdfmFile = TdfmFile::findOrFail($id);

        try {
            $fileName = $tdfmFile->name;
            $fileId = $tdfmFile->id;
            $tdfmFile->delete();
            Storage::disk('tdfm_generated')->deleteDir(DS . $fileId);
            Storage::disk('tdfm_original')->delete($fileName);
            app(CacheHelper::class)->removeFileData($id);

            if (file_exists(public_path('media/' . $tdfmFile->id))) {
                File::deleteDirectory(public_path('media/' . $tdfmFile->id), true);
            }

            return JsonResponse::create(['count' => $count]);
        } catch (Throwable $e) {
            Log::error($e);
            return JsonResponse::create($request->all(), 500);
        }
    }

    /**
     * @param $id
     * @param Request $request
     * @return JsonResponse
     */
    public function update($id, Request $request): JsonResponse
    {
        if (!isset($id) || !$request->has('original_name')) {
            return JsonResponse::create(['error' => 'no id or original_name']);
        }

        $tdfmFile = TdfmFile::findOrFail($id);
        try {
            $newName = Str::slug(
                pathinfo($request->get('original_name'), PATHINFO_FILENAME)
            ) . '.' . pathinfo($request->get('original_name'), PATHINFO_EXTENSION);
            $tdfmFile->original_name = $newName;
            $tdfmFile->update();

            app(SymlinkHelper::class)->symlinkFile($tdfmFile);
            app(CacheHelper::class)->removeFileData($id);
            return JsonResponse::create([]);
        } catch (Throwable $e) {
            Log::error($e);
            return JsonResponse::create($request->all(), 500);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function show(Request $request): JsonResponse
    {
        if (!$request->has('id')) {
            abort(404);
        }
        $size = !$request->has('size') ? 'xl' : $request->get('size');

        $response = PictureTag::pictureTag($request->get('id'), $size);
        return JsonResponse::create($response);
    }
}
